<?php

/**
 * Klutz_Driver:: defines an API for storing and retrieving the comic images
 *
 * $Horde: klutz/lib/Driver.php,v 1.16.2.1 2004/03/16 04:09:33 marcus Exp $
 *
 * @author  Marcus I. Ryan <marcus@riboflavin.net>
 * @version $Revision: 1.16.2.1 $
 * @since   Klutz 0.1
 * @package klutz
 */
class Klutz_Driver {

    /**
     * Attempts to return a concrete Klutz_Driver instance based on $driver.
     *
     * @param string    $driver     The type of concrete Klutz_Driver subclass
     *                              to return.  This is based on the storage
     *                              driver ($driver).  The code is dynamically
     *                              included.
     *
     * @param array     $params     (optional) A hash containing any additional
     *                              configuration or connection parameters a
     *                              subclass might need
     *
     * @return mixed    The newly created concrete Klutz_Driver instance, or
     *                  false on an error.
     */
    function &factory($driver, $params = array())
    {
        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        $driver = strtolower(basename($driver));

        if (is_null($params)) {
            if (is_null($GLOBALS['conf']['storage']['params'])) {
                $params = array();
            } else {
                $params = $GLOBALS['conf']['storage']['params'];
            }
        }

        if (empty($driver) || $driver == 'none') {
            return $ret = &new Klutz_Driver($params);
        }

        @include_once dirname(__FILE__) . '/Driver/' . $driver . '.php';
        $class = 'Klutz_Driver_' . $driver;
        if (class_exists($class)) {
            return $ret = &new $class($params);
        } else {
            return $ret = &new Klutz_Driver($params);
        }
    }

    /**
     * Attempts to return a reference to a concrete Klutz_Driver instance
     * based on $driver.  It wil only create a new instance if no
     * Klutz_Driver instance with the same parameters currently exists.
     *
     * This should be used if multiple storage sources are required.
     *
     * This method must be invoked as: $var = &Klutz_Driver::singleton()
     *
     * @param string    $driver     The type of concrete Klutz_Driver subclass
     *                              to return.  The is based on the storage
     *                              driver ($driver).  The code is dynamically
     *                              included.
     *
     * @param array     $params     (optional) A hash containing any additional
     *                              configuration or connection parameters a
     *                              subclass might need.
     *
     * @return mixed    The created concrete Klutz_Driver instance, or false
     *                  on error.
     */
    function &singleton($driver = null, $params = null)
    {
        static $instances;

        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        if (is_null($params)) {
            if (is_null($GLOBALS['conf']['storage']['params'])) {
                $params = array();
            } else {
                $params = $GLOBALS['conf']['storage']['params'];
            }
        }
        
        if (!isset($instances)) {
            $instances = array();
        }

        $signature = serialize(array($driver, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Klutz_Driver::factory($driver, $params);
        }

        return $instances[$signature];
    }

    function Klutz_Driver($params)
    {
    }

    /**
     * Get a list of the dates for which we have comics between $oldest and
     * $newest.  In the default driver (no backend) this is just a list of
     * all dates between $oldest and $newest.
     *
     * @param optional timestamp $date      The reference date (default today)
     * @param optional timestamp $oldest    The earliest possible date to
     *                                      return (default first of the month)
     * @param optional timestamp $newest    The latest possible date to return
     *                                      (default last date of the month)
     *
     * @return array timestamps             dates between $oldest and $newest
     *                                      we have comics for
     */
    function listDates($date = null, $oldest = null, $newest = null)
    {
        if (is_null($date)) {
            $date = mktime(0, 0, 0);
        }

        $dateparts = getdate($date);

        // Default to showing only the month specified.
        if (is_null($oldest)) {
            $oldest = mktime(0, 0, 0, $dateparts['mon'], 1, $dateparts['year']);
        }
        if (is_null($newest)) {
            $newest = mktime(0, 0, 0, $dateparts['mon'] + 1, 0, $dateparts['year']);
            $newest = min($newest, mktime(0, 0, 0));
        }

        $return = array();
        $i = date('j', $oldest);
        $loopMonth = date('n', $oldest);
        $loopYear = date('Y', $oldest);
        $loopStamp = mktime(0, 0, 0, $loopMonth, $i, $loopYear);
        while ($loopStamp <= $newest) {
            array_push($return, $loopStamp);
            $loopStamp = mktime(0, 0, 0, $loopMonth, ++$i, $loopYear);
        }

        return $return;
    }

    /**
     * Get the image dimensions for the requested image.  In the default
     * driver the image is not stored locally so this function returns
     * an empty string.  Performance hit is too expensive to make this
     * worth it.
     *
     * @param string $index             The index of the comic to check
     * @param optional timestamp $date  The date of the comic to check
     *                                  (default today)
     *
     * @return string           Attributes for an <img> tag giving height
     *                          and width
     */
    function imageSize($index, $date = null)
    {
        // Getting the image size is too expensive for the benefit
        // when using this driver.
        return '';
    }

    /**
     * Find out if we already have a local copy of this image.  In the
     * default driver we never store a local copy, so this is always false.
     *
     * @param string $index             The index of the comic to check
     * @param optional timestamp $date  The date of the comic to check
     *                                  (default today)
     *
     * @return boolean                  Returns false in this driver
     */
    function imageExists($index, $date = null)
    {
        return true;
    }

    /**
     * Store an image for later retrieval
     *
     * @param string $index             The index of the comic to retrieve
     * @param string $image             Raw (binary) image data to store
     * @param optional timestamp $data  Date to store it under (default today)
     *
     * @return boolean                  True on success, false otherwise
     */
    function storeImage($index, $image, $date = null)
    {
        return false;
    }

    /**
     * Retrieve an image from storage.  Since there is no local storage
     * this will actually call for the fetching.
     *
     * @param string $index             The index of the comic to retrieve
     * @param optional timestamp $date  The date for which we want $comic
     *
     * @return mixed                    If the image exists locally, return a
     *                                  Klutz_Image object.  If it doesn't,
     *                                  return a string with the URL pointing
     *                                  to the comic.
     */
    function retrieveImage($index, $date = null)
    {
        if (is_null($date)) {
            $date = mktime(0, 0, 0);
        }

        // For this driver, we grab the image on the fly
        global $klutz;
        $comic = $klutz->comicObject($index, $date);
        if (is_null($comic->referer) &&
             is_null($comic->agent) &&
             is_null($comic->user) &&
             is_null($comic->pass) &&
             count($comic->cookies) == 0 &&
             count($comic->headers) == 0) {
            return $comic->fetchURL($date);
        } else {
            return $comic->fetchImage($date);
        }
    }

}
